Info << "Field allocation for structure materials." << nl;

PtrList<volScalarField> Ta_sub(solidRegions.size());
PtrList<volScalarField> Tbc_sub(solidRegions.size());
PtrList<volScalarField> cp_sub(solidRegions.size());
PtrList<volScalarField> k_sub(solidRegions.size());
PtrList<volScalarField> rho_sub(solidRegions.size());
PtrList<volScalarField> fluxMap_sub(solidRegions.size());
PtrList<IOdictionary> subMatProperties(solidRegions.size());
PtrList<IOdictionary> dynamicMeshDict_sub(solidRegions.size());
RectangularMatrix<scalar> cp_sub_coefs(solidRegions.size(), 5);
RectangularMatrix<scalar> k_sub_coefs(solidRegions.size(), 5);
RectangularMatrix<scalar> rho_sub_coefs(solidRegions.size(), 5);
dimensionedTensor I_sub ("I_sub", dimless, tensor(1, 0, 0, 0, 1, 0, 0, 0, 1));

for (int i = 1; i < solidRegions.size(); i++)
{
    Info << "Allocating region: " << solidRegions[i].name() << nl;

    IOobject Ta_subIO
    (
        "Ta",
        runTime.timeName(),
        solidRegions[i],
        IOobject::MUST_READ,
        IOobject::AUTO_WRITE
    );

    if (Ta_subIO.headerOk())
    {
        Ta_sub.set
        (
            i,
            new volScalarField(Ta_subIO, solidRegions[i])
        );
    }
    else
    {
        Info << "Field Ta not found in region " << solidRegions[i].name() << nl;
        return (0);
    }

    IOobject k_subIO
    (
        "k_sub",
        runTime.timeName(),
        solidRegions[i],
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    );

    if (k_subIO.headerOk())
    {
        k_sub.set
        (
            i,
            new volScalarField(k_subIO, solidRegions[i])
        );
    }
    else
    {
        Info << "Field k_sub not found in region " << solidRegions[i].name()
             << nl;
        return (0);
    }

    subMatProperties.set
    (
        i,
        new IOdictionary
        (
            IOobject
            (
                "subMatProperties",
                runTime.constant(),
                solidRegions[i],
                IOobject::MUST_READ,
                IOobject::NO_WRITE
            )
        )
    );

    cp_sub.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "cp_sub",
                runTime.timeName(),
                solidRegions[i],
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            solidRegions[i],
            dimensionedScalar("1", dimensionSet(0, 2, -2, -1, 0, 0, 0), scalar(1.0))
        )
    );

    fluxMap_sub.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "fluxMap_sub",
                runTime.timeName(),
                solidRegions[i],
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            solidRegions[i],
            dimensionedScalar("1", dimless, scalar(1.0))
        )
    );

    rho_sub.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "rho_s",
                runTime.timeName(),
                solidRegions[i],
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            solidRegions[i],
            dimensionedScalar("1", dimensionSet(1, -3, 0, 0, 0, 0, 0), scalar(1.0))
        )
    );

    cp_sub_coefs[i][0] = subMatProperties[i].lookupOrDefault<scalar>("cp_sub_n[0]", 0.0);
    cp_sub_coefs[i][1] = subMatProperties[i].lookupOrDefault<scalar>("cp_sub_n[1]", 0.0);
    cp_sub_coefs[i][2] = subMatProperties[i].lookupOrDefault<scalar>("cp_sub_n[2]", 0.0);
    cp_sub_coefs[i][3] = subMatProperties[i].lookupOrDefault<scalar>("cp_sub_n[3]", 0.0);
    cp_sub_coefs[i][4] = subMatProperties[i].lookupOrDefault<scalar>("cp_sub_n[4]", 0.0);

    k_sub_coefs[i][0] = subMatProperties[i].lookupOrDefault<scalar>("k_sub_n[0]", 0.0);
    k_sub_coefs[i][1] = subMatProperties[i].lookupOrDefault<scalar>("k_sub_n[1]", 0.0);
    k_sub_coefs[i][2] = subMatProperties[i].lookupOrDefault<scalar>("k_sub_n[2]", 0.0);
    k_sub_coefs[i][3] = subMatProperties[i].lookupOrDefault<scalar>("k_sub_n[3]", 0.0);
    k_sub_coefs[i][4] = subMatProperties[i].lookupOrDefault<scalar>("k_sub_n[4]", 0.0);

    rho_sub_coefs[i][0] = subMatProperties[i].lookupOrDefault<scalar>("rho_sub_n[0]", 0.0);
    rho_sub_coefs[i][1] = subMatProperties[i].lookupOrDefault<scalar>("rho_sub_n[1]", 0.0);
    rho_sub_coefs[i][2] = subMatProperties[i].lookupOrDefault<scalar>("rho_sub_n[2]", 0.0);
    rho_sub_coefs[i][3] = subMatProperties[i].lookupOrDefault<scalar>("rho_sub_n[3]", 0.0);
    rho_sub_coefs[i][4] = subMatProperties[i].lookupOrDefault<scalar>("rho_sub_n[4]", 0.0);

// update Fields
    scalarField Ta_int = Ta_sub[i];

    forAll(cp_sub[i], cellI)
    {
        cp_sub[i][cellI] =
            cp_sub_coefs[i][0] +
            Ta_int[cellI] *
            (
                cp_sub_coefs[i][1] +
                Ta_int[cellI] *
                (
                    cp_sub_coefs[i][2] +
                    Ta_int[cellI] *
                    (
                        cp_sub_coefs[i][3] +
                        Ta_int[cellI] *
                        (
                            cp_sub_coefs[i][4]
                        )
                    )
                )
            );
    }

    forAll(k_sub[i], cellI)
    {
        k_sub[i][cellI] =
            k_sub_coefs[i][0] +
            Ta_int[cellI] *
            (
                k_sub_coefs[i][1] +
                Ta_int[cellI] *
                (
                    k_sub_coefs[i][2] +
                    Ta_int[cellI] *
                    (
                        k_sub_coefs[i][3] +
                        Ta_int[cellI] *
                        (
                            k_sub_coefs[i][4]
                        )
                    )
                )
            );
    }

    forAll(rho_sub[i], cellI)
    {
        rho_sub[i][cellI] =
            rho_sub_coefs[i][0] +
            Ta_int[cellI] *
            (
                rho_sub_coefs[i][1] +
                Ta_int[cellI] *
                (
                    rho_sub_coefs[i][2] +
                    Ta_int[cellI] *
                    (
                        rho_sub_coefs[i][3] +
                        Ta_int[cellI] *
                        (
                            rho_sub_coefs[i][4]
                        )
                    )
                )
            );
    }

    // read radiative patch name
    dynamicMeshDict_sub.set
    (
        i,
        new IOdictionary
        (
            IOobject
            (
                "dynamicMeshDict",
                runTime.constant(),
                solidRegions[i],
                IOobject::MUST_READ,
                IOobject::NO_WRITE
            )
        )
    );

}

Info << "Field allocation is done for the sub-structure." << nl;
