// read and store the B' table into the RAM for faster access
Info << "Reading the B' Table" << nl;

//  OFstream BprimeOutputFile ("BprimeOut");  // opens an output file
IFstream BprimeInputFile(
    "constant/environmentProperties/Bprime");  // opens an input file
IFstream BprimeInputFileTemp(
    "constant/environmentProperties/Bprime");  // opens an input file

if (BprimeInputFile.good() == false) // checks the input file is opened
{
    Info << "Bprime table not found" << nl;
    exit (1); // exits otherwise
}

Info << "The Bprime table must have 5 columns: p, B'g, B'c, T, Hg, ..." << nl;
int columnTableBp = 5; 
int rawTableBp = 0;
scalar tempBp, rawTableFracBp, rawTableIntBp;
int i_rawBp = 0;

while (true)
{
    BprimeInputFileTemp >> tempBp;
    if (BprimeInputFileTemp.eof() == 1) 
        break;
    i_rawBp++;
}
rawTableFracBp = 
    modf
    (
        static_cast<scalar>(i_rawBp) / 
        static_cast<scalar>(columnTableBp), 
        &rawTableIntBp
    );

if (rawTableFracBp != 0) 
{
    Info << "... and it does not seem to have 5 columns." << nl;
    Info << rawTableIntBp << " lines and " 
         << rawTableFracBp * columnTableBp << " 'scalar' have been read." 
         << nl;
    exit(1);
}
else 
{
    rawTableBp = i_rawBp / columnTableBp;
    Info << "The Bprime table has been read (" << rawTableBp << " lines)." 
         << nl;
}

RectangularMatrix<scalar> Table(rawTableBp, columnTableBp);

for (int x = 0; x < rawTableBp; x++)
{
    for (int i = 0; i < columnTableBp; i++)
    {
        BprimeInputFile >> Table[x][i];
        // BprimeOutputFile << Table[x][i] << "    ";
    }
    // BprimeOutputFile << nl;
}

// BprimeT table is an object of the class BprimeSI, which handles the B' table interpolations when called
BprimeSI BprimeT(Table);

Info << "The B' Table has been read" << nl;
//   Info << "Process killed after reading the B' Table" << nl; return (0);  // exits without error
// END OF THE IO TEST
