// read and store the gas-property table into the RAM for faster access
Info << "Reading the gas property table" << nl;

//  OFstream gasPropertyOutputFile ("gasPropertiesOut");  // opens an output file
IFstream gasPropertyInputFile(
    "constant/porousMat/gasProperties");  // opens an input file
IFstream gasPropertyInputFileTemp(
    "constant/porousMat/gasProperties");  // opens an input file

if (gasPropertyInputFile.good() == false)
{
    Info << "gasProperties file not found" << nl;
    exit(1); // exits otherwise
}

Info << "The gas-property table must have 5 columns: p, T, M, h, nu" 
     << nl;
int columnTableGP = 5; 
int rawTableGP = 0;
scalar tempGP, rawTableFracGP, rawTableIntGP;
int i_rawGP = 0;

while (true)
{
    gasPropertyInputFileTemp >> tempGP;
    if (gasPropertyInputFileTemp.eof() == 1) 
        break;
    i_rawGP++;
}

rawTableFracGP = 
    modf
    (
        static_cast<scalar>(i_rawGP) / 
        static_cast<scalar>(columnTableGP), 
        &rawTableIntGP
    );

if (rawTableFracGP != 0) 
{
    Info << "... and it does not seem to have 5 columns." << nl;
    Info << rawTableIntGP << " lines and " 
         << rawTableFracGP * columnTableGP << " 'scalar' have been read." 
         << nl;
    exit(1);
}
else
{
    rawTableGP = i_rawGP / columnTableGP;
    Info << "The gas-property table has been read (" << rawTableGP << " lines)." 
         << nl;
}

RectangularMatrix<scalar> TableGP(rawTableGP, columnTableGP);
for (int x = 0; x < rawTableGP; x++)
{
    for (int i = 0; i < columnTableGP; i++)
    {
        gasPropertyInputFile >> TableGP[x][i];
        // gasPropertyOutputFile << TableGP[x][i] << " ";
    }
    // gasPropertyOutputFile << nl;
}

// "gasPT" is an object of the class gasProperties, which handles the gas property lookup and interpolations when called
gasProperties gasPT(TableGP);

// Tests 8/3/17
/*
Info << "M(P1) =" << TableGP[0][0] << nl ;
Info << "M(P=max) =" << TableGP[rawTableGP-1][0] << nl ;

Info << "TableGP.n() = " << TableGP.n() << nl ;
Info << "TableGP.m() = " << TableGP.m() << nl ;
*/
// return(0);

Info << "The gasPropertyTable has been read" << nl;
//   Info << "Process killed after reading the gasPropertyTable" << nl; return (0);  // exits without error
// END OF THE IO TEST
