PtrList<scalarField> fluxMapPtr(solidRegions.size());

if (axiSymmetricalFlux)
{
    // read and store the boundary layer properties into the RAM for faster access
    Info << "Reading flux Map" << nl;

    // OFstream fluxFactorMapOutputFile ("fluxFactorMapOutputFile");  // opens an output file
    IFstream fluxFactorMap("constant/environmentProperties/fluxFactorMap");  // opens an input file
    IFstream fluxFactorMapTemp("constant/environmentProperties/fluxFactorMap");  // opens a temporary input file

    if (fluxFactorMap.good() == false) // checks that the input file is opened
    {
        Info << "fluxFactorMap not found" << nl;
        exit (1); // exits otherwise
    }

    Info << "The fluxFactorMap must have 3 columns: "
         "mapping variable (e.g. Y), flux factor, pressure factor"
         << nl;

    int columnTableFF = 3;
    int rawTableFF = 0;
    scalar tempFF, rawTableFracFF, rawTableIntFF;
    int i_rawFF = 0;

    while (true)
    {
        fluxFactorMapTemp >> tempFF;
        if (fluxFactorMapTemp.eof() == 1) {break;}
        i_rawFF++;
    }

    rawTableFracFF = modf(static_cast<scalar>(i_rawFF) / static_cast<scalar>(columnTableFF), &rawTableIntFF);
    if (rawTableFracFF != 0)
    {
        Info << "... and it does not seem to have 3 columns." << nl;
        Info << rawTableIntFF << " lines and "
             << rawTableFracFF * columnTableFF
             << " 'double precision' have been read."
             << nl;
        exit(1);
    }
    else
    {
        rawTableFF = i_rawFF / columnTableFF;
        Info << "The fluxFactorMap has been read (" << rawTableFF << " lines)." << nl;
    }


    RectangularMatrix<scalar> fluxFactorTable(rawTableFF, columnTableFF);
    for (int x = 0; x < rawTableFF; x++)
    {
        for (int i = 0; i < columnTableFF; i++)
        {
            fluxFactorMap >> fluxFactorTable[x][i];
            // fluxFactorMapOutputFile << fluxFactorTable[x][i] << "   ";
        }
        // fluxFactorMapOutputFile << nl;
    }

    // fluxFactorAxi is an object of the class fluxFactor, which handles the interpolation for the flux factor as a function of distance when called - please refer to the class fluxFactor for more details
    fluxFactor fluxFactorAxi(fluxFactorTable);
    // the call format is fluxFactorAxi.value(distance);
    //  Info << "factor1 = " << fluxFactorAxi.value(0.03) << nl;
    //  Info << "The flux mapping factors have been read." << nl;
    //  Info << "Process killed after reading the flux factor map." << nl; return (0);  // exits without error
    // END OF THE IO TEST

    // Compute the flux factor (or transfer coefficient factor) for the faces of the ablating suface for 2D and 3D axi-symmetrical heat fluxes
    vectorField c = mesh.Cf().boundaryField()[movingPatchID] - fluxFactorCenter;
    scalarField Distance = (c & fluxFactorNormal);

    if (fluxFactorProjection)
    {
        // distance to center, on the plane perpendicular to fluxFactorDirection
        // (assuming that fluxFactorNormal is either vectorX, vectorY, or vectorZ,
        // as required in the input file).
        Distance =
            sqrt
            (
                (c & vectorX) * (c & vectorX)
              + (c & vectorY) * (c & vectorY)
              + (c & vectorZ) * (c & vectorZ)
              - Distance * Distance
            );
    }

    forAll(fluxMap.boundaryField()[movingPatchID], faceI)
    {
        fluxMap.boundaryFieldRef()[movingPatchID][faceI] =
            fluxFactorAxi.fluxValue(Distance[faceI]);
        pressureMap.boundaryFieldRef()[movingPatchID][faceI] =
            fluxFactorAxi.pressureValue(Distance[faceI]);
    }

    // Compute ... idem ... for subMati ...
    for (int i = 1; i < solidRegions.size(); i++)
    {
        // read patch name
        word externalPatchName(dynamicMeshDict_sub[i].lookup("externalPatch"));
        label externalPatchID = solidRegions[i].boundaryMesh().findPatchID(externalPatchName);

        if (externalPatchID < 0)
        {
            Info << "subMat external Patch Not Found" << nl;
        }
        else
        {
            vectorField c =
                solidRegions[i].Cf().boundaryField()[externalPatchID] - fluxFactorCenter;
            scalarField Distance = (c & fluxFactorNormal);

            if (fluxFactorProjection)
            {
                // distance to center, on the plane perpendicular to fluxFactorDirection
                // (assuming that fluxFactorNormal is either vectorX, vectorY, or vectorZ,
                // as required in the input file).
                Distance =
                    sqrt
                    (
                        (c & vectorX) * (c & vectorX)
                      + (c & vectorY) * (c & vectorY)
                      + (c & vectorZ) * (c & vectorZ)
                      - Distance * Distance
                    );
            }

            forAll(fluxMap_sub[i].boundaryField()[externalPatchID], faceI)
            {
                fluxMap_sub[i].boundaryFieldRef()[externalPatchID][faceI] =
                    fluxFactorAxi.fluxValue(Distance[faceI]);
            }
        }
    }
//  scalar DistanceMax = Distance[0];  -- development only (3 following lines)
//  forAll(Distance, faceI) {if (Distance[faceI] > DistanceMax) {DistanceMax = Distance[faceI];}}
//  forAll(fluxMap.boundaryField()[movingPatchID], faceI)
//  {fluxMap.boundaryField()[movingPatchID][faceI]= fluxFactorAxi.value(DistanceMax-Distance[faceI]);}
} // end if(axiSymmetricalFlux)

else // if (axiSymmetricalFlux)
{
    // factor = 1
    scalar one_scalar = 1.0;

    forAll(fluxMap.boundaryField()[movingPatchID], faceI)
    {
        fluxMap.boundaryFieldRef()[movingPatchID][faceI] = one_scalar;
    }

    forAll(pressureMap.boundaryField()[movingPatchID], faceI)
    {
        pressureMap.boundaryFieldRef()[movingPatchID][faceI] = one_scalar;
    }

    // Compute ... idem ... for subMati ...
    for (int i = 1; i < solidRegions.size(); i++)
    {
        // read patch name
        word externalPatchName(dynamicMeshDict_sub[i].lookup("externalPatch"));
        label externalPatchID =
            solidRegions[i].boundaryMesh().findPatchID(externalPatchName);

        if (externalPatchID < 0)
        {
            Info << "subMat external Patch Not Found" << nl;
        }
        else
        {
            forAll(fluxMap_sub[i].boundaryField()[externalPatchID], faceI)
            {
                fluxMap_sub[i].boundaryFieldRef()[externalPatchID][faceI] =
                    one_scalar;
            }
        }
    }
}
