/*---------------------------------------------------------------------------*\
YEqn.H

Description
    Species conservation equation (finite-rate chemistry mode).
\*---------------------------------------------------------------------------*/

epsphi = mDotGface & mesh.Sf();           // Mass flux of pyrolysis gases.
epsrhog = eps_g * rho_g;                  // Volume-averaged gas density.

tmp<fv::convectionScheme<scalar> > mvConvection
(
    fv::convectionScheme<scalar>::New
    (
        mesh,
        fields,
        epsphi,
        mesh.divScheme("div(epsphi,Yi)")
    )
);

{
    // label inertIndex = -1;
    volScalarField Yt = 0.0 * Y[0];
    for (label i = 0; i < Y.size(); i++)
    {
        if (Y[i].name() != "C(S)")
        {
            volScalarField& Yi = Y[i];                  // Species i mass fraction field
            volScalarField& Dmi = Dm[i];                // Species i average multicomponent diffusion coefficient field     Yini = Y[i].oldtime();
            volScalarField totalYp = piTotal;
            totalYp = totalYp * 0;

            forAll(Xsi, j)
            {
                totalYp += Yp[j][i] * pi[j];
            }

            solve
            (
                  fvm::ddt(epsrhog, Yi) -                            // storage
                - fvm::div(fvc::interpolate(epsrhog)*mesh.phi(), Yi) // can be neglected because mesh motion slow compared to chemistry time scale and gas velocity
                + mvConvection->fvmDiv(epsphi, Yi)                   // convection
                - fvm::laplacian(Dmi / eta0 * epsrhog, Yi)           // fickian diffusion
                - totalYp                                            // species produced by the pyrolysis reactions
                ==
                  eps_g * chemistry.RR(i),                           // finite-rate chemistry production (computed in 'chemistry.H')
                mesh.solver("Yi")
            );

            Yi.max(0.0);
            Yt += Yi;
        }
    }

    for (label i = 0; i < Y.size(); i++)
    {
        Y[i] = Y[i] / Yt;
    }
}

// compute the energy (flux) transported by diffusion of the molecules
Ediff = Ediff0;

// speciesHOverRT(Ta, hSpeciesCell)
for (label i = 0; i < Y.size(); i++)
{
    volScalarField& Dmi = Dm[i];
    volScalarField& hSpeciesFieldi = hSpeciesField[i];

    // energy (enthalpy) transported by molecular diffusion
    Ediff = Ediff + fvc::laplacian(Dmi / eta0 * epsrhog * hSpeciesFieldi, Y[i]);
}

/* to print the enthalpy of the mixture in J/kg as a function of T (in K)
for (int tvar = 0 ; tvar < 20 ; tvar ++ ){
scalar enth=0;
    for(label i=0; i<Y.size(); i++)
    {
enth = enth + Y[i][0] * chemistry.hsi(i, 200 + tvar *100);
    }
Info << "h[T= " << 200 + tvar * 100 << "] =" << enth << nl;
}
*/

/*** Compute new delatT  ***/

// set tolerance on mass fraction variation rate
scalar Ythreshold = 1e-3;
if (runTime.controlDict().found("Ythreshold"))
{
    Ythreshold = readScalar(runTime.controlDict().lookup("Ythreshold"));
}

scalar dYtolMin = 0.01;
if (runTime.controlDict().found("dYtolMin"))
{
    dYtolMin = readScalar(runTime.controlDict().lookup("dYtolMin"));
}

scalar dYtolMax = 0.05;
if (runTime.controlDict().found("dYtolMax"))
{
    dYtolMax = readScalar(runTime.controlDict().lookup("dYtolMax"));
}

volScalarField Yini = 0.0 * Y[0];
scalar dYmax = 0;
scalar speciesMax = 0;
scalar cellMax = 0;

for (label i = 0; i < Y.size(); i++)
{
    if (Y[i].name() != "C(S)")
    {
        Yini = Y[i].oldTime();
        forAll(Yini, celli)
        {
            if (Yini[celli] > Ythreshold)
            {
                if (fabs((Yini[celli] - Y[i][celli]) / Yini[celli])  > dYmax)
                {
                    dYmax = fabs((Yini[celli] - Y[i][celli]) / Yini[celli]);
                    speciesMax = i; cellMax = celli;
                }
            }
        }
    }
}

Info << "dYmax(species: " << Y[speciesMax].name()
     << "; in cell: " << cellMax
     << ") = " << dYmax
     << " | Y = " << Y[speciesMax][cellMax]
     << nl;

if (dYmax > dYtolMax)
{
    dtChemYEqn = runTime.deltaT().value() / 1.2;
}

if (dYmax < dYtolMin)
{
    dtChemYEqn = runTime.deltaT().value() * 1.2;
}
