/**** Initialization ****/

Switch adjustTimeStep
(
    runTime.controlDict().lookup("adjustTimeStep")
);

scalar maxCo
(
    readScalar(runTime.controlDict().lookup("maxCo"))
);

scalar maxDeltaT = GREAT;

if (runTime.controlDict().found("maxDeltaT"))
{
    maxDeltaT = readScalar(runTime.controlDict().lookup("maxDeltaT"));
}

scalar REVlength = 1e-3;

if (runTime.controlDict().found("REVlength"))
{
    REVlength = readScalar(runTime.controlDict().lookup("REVlength"));
}

scalar minDeltaT = SMALL;

if (runTime.controlDict().found("minDeltaT"))
{
    minDeltaT = readScalar(runTime.controlDict().lookup("minDeltaT"));
}

/**** Set "CFL" number ****/

scalar CoNum = 0.0;
scalar meanCoNum = 0.0;
scalar velMag = 0.0;
kS = linearInterpolate(k);
phiT = kS & (fvc::snGrad(Ta) * mesh.Sf() / mesh.magSf());

Info << "max(mag(phiT)) = "
     << max(mag(phiT))
     << nl;

if (mesh.nInternalFaces())
{
    surfaceScalarField SfUfbyDelta =
        mag(phiT) / REVlength;

    CoNum = max(SfUfbyDelta).value() * runTime.deltaT().value();

    meanCoNum =
        (
            sum(SfUfbyDelta) /
            sum(mesh.magSf())
        ).value() *
        runTime.deltaT().value();

    velMag = max(mag(phiT)).value();

    /*
        CoNum = max(SfUfbyDelta/mesh.magSf())
            .value()*runTime.deltaT().value();

        meanCoNum = (sum(SfUfbyDelta)/sum(mesh.magSf()))
            .value()*runTime.deltaT().value();

        velMag = max(mag(vS)/mesh.magSf()).value();
    */
}

Info << "Courant Number mean: " << meanCoNum
     << " max: " << CoNum
     << " velocity magnitude: " << velMag << endl;

// ***** Adjust time step ***** //

if (adjustTimeStep)
{
    scalar maxDeltaTFact = maxCo / (CoNum + SMALL);
    // scalar deltaTFact = min(min(maxDeltaTFact, 1.0 + 0.1*maxDeltaTFact), 1.2);
    scalar deltaTFact = min(min(maxDeltaTFact, 1.0 + 0.5 * maxDeltaTFact), 10);

    Info << "maxDeltaTFact initial = " << maxDeltaTFact << nl;
    Info << "deltaTFact damped = " << deltaTFact << nl;

    runTime.setDeltaT
    (
        max(
            min
            (
                deltaTFact * runTime.deltaT().value(),
                maxDeltaT
            )
            ,
            minDeltaT
        )
    );

    Info << "deltaT = " <<  runTime.deltaT().value() << endl;
}
