// Compute gas properties - either reading tables or using Mutation++ (depending on options).

if (finiteRate && mutationTT)
{
    forAll (M, cellI)
    {
        // initialisation of the pointers for T,p (pTp) and for the species (pY), used in setState(pY,pTp,2) in Mutation++
        pTp[1] = Tg[cellI];
        pTp[0] = p[cellI];

        for (int i = 0; i < ns; i++)
        {
            pY[speciesIndexMutation[i]] = Y[i][cellI];
        }

        // set mixture composition in mutation++
        mixFiniteRate.setState(pY, pTp, 2);

        // compute thermochemical data
        M[cellI]  = mixFiniteRate.mixtureMw();
        hg[cellI] = mixFiniteRate.mixtureHMass();
        mu[cellI] = mixFiniteRate.viscosity();

        if (multiComponent)
        {
            mixFiniteRate.averageDiffusionCoeffs(Ds);
            for (int i = 0; i < ns; i++)
            {
                Dm[i][cellI] = Ds[speciesIndexMutation[i]];
            }
        }

        mixFiniteRate.speciesHOverRT(Tg[cellI], hSpecies);
        for (int i = 0; i < ns; i++)
        {
            hSpeciesField[i][cellI] =
                hSpecies[speciesIndexMutation[i]] * R.value() * Tg[cellI];
        }
    }

    forAll (M.boundaryField()[movingPatchID], faceI)
    {
        // initialisation of p,T (pTp) and species (pY) pointers for setState(pY,pTp,2) of mutation++
        pTp[1] = Tg.boundaryField()[movingPatchID][faceI];
        pTp[0] = p.boundaryField()[movingPatchID][faceI];

        for (int i = 0; i < ns; i++)
        {
            pY[speciesIndexMutation[i]] = Y[i].boundaryField()[movingPatchID][faceI];
        }

        mixFiniteRate.setState(pY, pTp, 2);
        M.boundaryFieldRef()[movingPatchID][faceI]  = mixFiniteRate.mixtureMw();
        hg.boundaryFieldRef()[movingPatchID][faceI] = mixFiniteRate.mixtureHMass();
        mu.boundaryFieldRef()[movingPatchID][faceI] = mixFiniteRate.viscosity();

        if (multiComponent)
        {
            mixFiniteRate.averageDiffusionCoeffs(Ds);
            for (int i = 0; i < ns; i++)
            {
                Dm[i].boundaryFieldRef()[movingPatchID][faceI] = Ds[speciesIndexMutation[i]];
            }
        }

        mixFiniteRate.speciesHOverRT(Tg.boundaryField()[movingPatchID][faceI], hSpecies);
        for (int i = 0; i < ns; i++)
        {
            hSpeciesField[i].boundaryFieldRef()[movingPatchID][faceI] =
                hSpecies[speciesIndexMutation[i]] *
                R.value() *
                Tg.boundaryField()[movingPatchID][faceI];
        }
    }

}
else // Equilibrium chemistry
{
    if (mutationTT) // Computed with Mutation++
    {
        if (!elementConservation)
        {
            scalar T_var = BL.Tinf(runTime.value());
            scalar p_var = BL.pw(runTime.value());

            if (!readBoundaryTecplotTable)
            {
                pTp[1] = T_var; pTp[0] = p_var;
                mix.setState(p_Zx, pTp, 2);
            }
            forAll (M, i)
            {
                if (!readBoundaryTecplotTable)
                {
                    if
                    (
                        (T_var > Tg[i] + 10.) ||
                        (T_var < Tg[i] - 10.) ||
                        (p_var > min((p[i] +  p[i] / 10.), (p[i] + 1000.))) ||
                        (p_var < max((p[i] - p[i] / 10.), (p[i] - 1000.)))
                    )
                    {
                        T_var  = Tg[i];
                        p_var  = p[i];
                        pTp[1] = T_var;
                        pTp[0] = p_var;
                        // for(int i=0; i<ne_mix; i++)  { Info << "Z[" << mix.elementName(i) <<"] = " << p_Zx[i] << nl;}
                        mix.setState(p_Zx, pTp, 2);
                    }
                }
                else
                {
                    T_var  = Tg[i];
                    p_var  = p[i];
                    pTp[1] = T_var;
                    pTp[0] = p_var;
                    mix.setState(p_Zx, pTp, 2);
                }

                M[i]  = mix.mixtureMw();
                hg[i] = mix.mixtureHMass();
                mu[i] = mix.viscosity();
            }

            forAll(M.boundaryField()[exitPatchID], faceI)
            {
                if (!readBoundaryTecplotTable)
                {
                    if
                    (
                        (T_var > Tg.boundaryField()[exitPatchID][faceI] + 10.) ||
                        (T_var < Tg.boundaryField()[exitPatchID][faceI] - 10.) ||
                        (p_var > p.boundaryField()[exitPatchID][faceI] + 10.) ||
                        (p_var < p.boundaryField()[exitPatchID][faceI] - 10.)
                    )
                    {
                        T_var  = Tg.boundaryField()[exitPatchID][faceI];
                        p_var  = p.boundaryField()[exitPatchID][faceI];
                        pTp[1] = T_var;
                        pTp[0] = p_var;
                        mix.setState(p_Zx, pTp, 2);
                    }
                }
                else
                {
                    T_var  = Tg.boundaryField()[exitPatchID][faceI];
                    p_var  = p.boundaryField()[exitPatchID][faceI];
                    pTp[1] = T_var;
                    pTp[0] = p_var;
                    mix.setState(p_Zx, pTp, 2);
                }
                M.boundaryFieldRef()[exitPatchID][faceI]  = mix.mixtureMw();
                hg.boundaryFieldRef()[exitPatchID][faceI] = mix.mixtureHMass();
                mu.boundaryFieldRef()[exitPatchID][faceI] = mix.viscosity();
            }

            forAll(M.boundaryField()[movingPatchID], faceI)
            {
                if (!readBoundaryTecplotTable)
                {
                    if
                    (
                        (T_var > Tg.boundaryField()[movingPatchID][faceI] + 10.) ||
                        (T_var < Tg.boundaryField()[movingPatchID][faceI] - 10.) ||
                        (p_var > p.boundaryField()[movingPatchID][faceI] + 10.) ||
                        (p_var < p.boundaryField()[movingPatchID][faceI] - 10.)
                    )
                    {
                        T_var  = Tg.boundaryField()[movingPatchID][faceI];
                        p_var  = p.boundaryField()[movingPatchID][faceI];
                        pTp[1] = T_var;
                        pTp[0] = p_var;
                        mix.setState(p_Zx, pTp, 2);
                    }
                }
                else
                {
                    T_var  = Tg.boundaryField()[movingPatchID][faceI];
                    p_var  = p.boundaryField()[movingPatchID][faceI];
                    pTp[1] = T_var;
                    pTp[0] = p_var;
                    mix.setState(p_Zx, pTp, 2);
                }

                M.boundaryFieldRef()[movingPatchID][faceI]  = mix.mixtureMw();
                hg.boundaryFieldRef()[movingPatchID][faceI] = mix.mixtureHMass();
                mu.boundaryFieldRef()[movingPatchID][faceI] = mix.viscosity();
            }
        } // end (!elementConservation)
        else // if (elementConservation)
        {
            forAll (M, cellI)
            {
                pTp[1] = Tg[cellI];
                pTp[0] = p[cellI];

                for (int i = 0; i < ne_mix; i++)
                {
                    p_Zx[i] = Z[i][cellI];
                }
                // for(int i=0; i<ne_mix; i++)  { Info << "Z[" << mix.elementName(i) <<"] = " << p_Zx[i] << nl;}
                mix.convert<Mutation::Thermodynamics::YE_TO_XE>(p_Zx, p_Zx);
                mix.setState(p_Zx, pTp, 2);
                M[cellI]  = mix.mixtureMw();
                hg[cellI] = mix.mixtureHMass();
                mu[cellI] = mix.viscosity();
            }

            forAll(M.boundaryField()[exitPatchID], faceI)
            {
                pTp[1] = Tg.boundaryField()[exitPatchID][faceI];
                pTp[0] = p.boundaryField()[exitPatchID][faceI];

                for (int i = 0; i < ne_mix; i++)
                {
                    p_Zx[i] = Z[i].boundaryField()[exitPatchID][faceI];
                }
                mix.convert<Mutation::Thermodynamics::YE_TO_XE>(p_Zx, p_Zx);
                mix.setState(p_Zx, pTp, 2);
                M.boundaryFieldRef()[exitPatchID][faceI]  = mix.mixtureMw();
                hg.boundaryFieldRef()[exitPatchID][faceI] = mix.mixtureHMass();
                mu.boundaryFieldRef()[exitPatchID][faceI] = mix.viscosity();
            }

            forAll(M.boundaryField()[movingPatchID], faceI)
            {
                pTp[1] = Tg.boundaryField()[movingPatchID][faceI];
                pTp[0] = p.boundaryField()[movingPatchID][faceI];

                for (int i = 0; i < ne_mix; i++)
                {
                    p_Zx[i] = Z[i].boundaryField()[movingPatchID][faceI];
                }
                mix.convert<Mutation::Thermodynamics::YE_TO_XE>(p_Zx, p_Zx);
                mix.setState(p_Zx, pTp, 2);
                M.boundaryFieldRef()[movingPatchID][faceI]  = mix.mixtureMw();
                hg.boundaryFieldRef()[movingPatchID][faceI] = mix.mixtureHMass();
                mu.boundaryFieldRef()[movingPatchID][faceI] = mix.viscosity();
            }
        } // end (equilibriumConservation)
    } // end mutationTT

    else // (if !mutationTT then) Read from table
    {
        forAll (M, i)
        {
            M[i] = gasPT.M(p[i], Tg[i]);
        }

        forAll (hg, i)
        {
            hg[i] = gasPT.h(p[i], Tg[i]);
        }

        forAll (mu, i)
        {
            mu[i] = gasPT.mu(p[i], Tg[i]);
        }

        forAll(M.boundaryField()[movingPatchID], faceI)
        {
            M.boundaryFieldRef()[movingPatchID][faceI] =
                gasPT.M
                (
                    p.boundaryField()[movingPatchID][faceI],
                    Tg.boundaryField()[movingPatchID][faceI]
                );
        }
        forAll(hg.boundaryField()[movingPatchID], faceI)
        {
            hg.boundaryFieldRef()[movingPatchID][faceI] =
                gasPT.h
                (
                    p.boundaryField()[movingPatchID][faceI],
                    Tg.boundaryField()[movingPatchID][faceI]
                );
        }
        forAll(mu.boundaryField()[movingPatchID], faceI)
        {
            mu.boundaryFieldRef()[movingPatchID][faceI] =
                gasPT.mu
                (
                    p.boundaryField()[movingPatchID][faceI],
                    Tg.boundaryField()[movingPatchID][faceI]
                );
        }
    }
}
